#!/bin/bash
####################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development L.P.
# @(#) HPE Serviceguard Oracle ASM MIRROR Utility Script.
# @(#) Product Name                :  serviceguard-oracle-toolkit
# @(#) Product Version             :  A.12.20.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HPE Serviceguard Oracle Toolkit
#
# Changing this file may lead to unrecoverable Serviceguard Oracle
# toolkit package startup and/or shutdown problems.
#
####################################################################
# Utility functions file used by the ASM mirror utility script
####################################################################
SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
export SG_UTILS

# Set PATH to reference the appropriate directories.
PATH=${SGSBIN}:/bin:/sbin:/usr/bin:/usr/sbin

####################################################################
# Source in the utility functions used by the module scripts
####################################################################
if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility "\
             "functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility "\
         "functions file: ${SG_UTILS}"
    exit 1
fi

sg_source_pkg_env $*

# OS command micro 
UDEV_INFO="udevadm info -q name -n"
UDEV_PROP="udevadm info -q property -n"
DM_DETAILS="dmsetup info"
MPATH_LIST="multipath -ll"

# OS command constants
cmexec=${SGSBIN}/cmexec
cmmodpkg=${SGSBIN}/cmmodpkg
cmrunpkg=${SGSBIN}/cmrunpkg
cmviewcl=${SGSBIN}/cmviewcl 

# Global variable describing different state's value

export ORA_DB_SID=${ORACLE_SID_NAME}
export ORA_HOME=${ORACLE_ASM_HOME}
export ORA_ADMIN=${ORACLE_ASM_USER}
export ORA_SID=${ORACLE_ASM_SID}
export ORA_ASM=${ORACLE_ASM}
export ORA_INSTANCE_TYPE=$ORACLE_INSTANCE_TYPE

# Global variable to hold the output of cmexec
declare -r OFFLINE=0
declare -r ONLINE=1
declare -r TRUE=0
declare -r FALSE=1
typeset -i sgsadta_cmexec_err=255
typeset -i NA=-1
typeset -i cmexec_retry_interval
typeset ASM_PV_LIST
typeset ASM_DISK_INFO 
typeset total_sitenodes
typeset ASM_DG=""
typeset ASM_FG=""
typeset ASM_DSK_NAME=""
typeset ASM_DEV_PATH=""
typeset ASM_DEV=""
typeset CURRENT_PKG=${SG_PACKAGE_NAME}
typeset ORA_ASM_MIR_LOG_DIR="${ORACLE_ORACLE_HOME}/log"
typeset ORA_ASM_MIR_LOG_FILE="${ORA_ASM_MIR_LOG_DIR}/.ora_asm_mir_log_${ORA_DB_SID}"
typeset ORA_ASM_MIR_STATUS_DIR="${SGCONF}/scripts/tkit/oracle/${CURRENT_PKG}"
typeset DG_STATUS_FILE="${ORA_ASM_MIR_STATUS_DIR}/dg_status_file_${ORA_DB_SID}"
typeset ASM_DRIVER_INFO_FILE="${ORA_ASM_MIR_STATUS_DIR}/.ora_asm_driver_info_${ORA_DB_SID}"
typeset ORA_ASM_MIR_CONF_FILE="${ORA_ASM_MIR_STATUS_DIR}/.ora_asm_mir_conf_${ORA_DB_SID}"
typeset TMP_DG_STATUS_FILE="${ORA_ASM_MIR_LOG_DIR}/tmp_dg_status_file_${ORA_DB_SID}"
typeset ORA_ASM_MIR_EMAIL_FILE="${ORA_ASM_MIR_LOG_DIR}/.email_body_$$_${ORA_DB_SID}"
typeset HOST=`hostname | cut -d. -f 1`
typeset monitoring_service_name
typeset monitoring_service_cmd
typeset ORACLEASM="/usr/sbin/oracleasm"
typeset ORACLEASM_LIB_DISKS="/dev/oracleasm/disks";
typeset NUM_OF_FG_IN_DG
typeset TYPE_OF_MSG="WARNING"
typeset ERROR_MSG=""
typeset ORA_ASM_DRIVER_TYPE=""

#"IS_FG_SYNC_IN_DG" variable value will be used for RPO criteria check only 
#if DG status file does not exist.
#If DG status file exists "IS_FG_SYNC_IN_DG" value will be always set to "FALSE" 
#and RPO criteria check will be performed with "CLEAN_FG_IN_DG" flag value.
typeset IS_FG_SYNC_IN_DG  
#"CLEAN_FG_IN_DG" variable value will be used for RPO criteria only if DG 
#status file exist.
typeset CLEAN_FG_IN_DG  
typeset cmviewcl_output=`$SGSBIN/cmviewcl -vf line -s config 2>/dev/null`
typeset VMFS_MODULE="sg/vmfs"

typeset monitoring_service_name
typeset monitoring_service_cmd

###############################################################################
# This function deletes the temporary command line log file 
# Internal Use only
#
#
# Arguments:  None
#
# 
# Returns:    None
#
###############################################################################
function cleanup_temp_cmd_log_file 
{
    sg_log 5 "Function cleanup_temp_cmd_log_file"
    typeset tmpfile=${ORA_ASM_MIR_LOG_FILE}
    if [[ -n "$1" ]];then
        tmpfile=$1
    fi 
    if [[ -f $tmpfile ]]; then
        rm -f $tmpfile >/dev/null 2>&1
    fi
    return 
}

###############################################################################
# This function creates the temporary command line log file, if file exists
# then deletes the file and recreates.
# Internal Use only
#
#
# Arguments:  None
#
#
# Returns:    None
#
###############################################################################
function recreate_tmp_cmd_log_file 
{
    sg_log 5 "Function recreate_tmp_cmd_log_file"
    typeset tmpfile=${ORA_ASM_MIR_LOG_FILE}
    cleanup_temp_cmd_log_file
    `touch $tmpfile 2>/dev/null` >/dev/null 2>&1
    return
}

###############################################################################
# This function caches DISKGROUP
# Internal Use only
#
#
# Arguments:  None
#
#
# Returns:    1 for error, 0 for success 
#
###############################################################################
function cache_dg_info 
{
    sg_log 5 "Function cache_dg_info"
    typeset -i dg_index=0
    for dg in "${ORACLE_ASM_DISKGROUP[@]}"
    do
        cache_dg_fg_info $dg $dg_index >/dev/null 2>&1
        if [[ $? -ne 0 ]];then 
            return 1
        fi
        dg_index=$(($dg_index+1))
    done
    return 0    
}

###############################################################################
# This function caches FAILGROUP and respective ASM_DEVICE name
# Internal Use only
#
#
# Arguments:  failgroup name and diskgroup name 
#
#
# Returns:    1 for error, 0 for success 
#
###############################################################################
function cache_fg_sync_info
{
    sg_log 5 "Function cache_fg_sync_info"
    typeset fg_name=$1
    typeset dg_name=$2
    typeset dg_index=$3
    typeset -i exit_code=0
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select d.MODE_STATUS from v\\\$asm_diskgroup dg, v\\\$asm_disk d 
            where dg.group_number=d.group_number AND dg.name='$dg_name' AND 
            d.FAILGROUP='$fg_name';"
    result=\`echo "\$sqltext"| \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
    `cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |\
                            grep -q "MODE_ST"` >/dev/null 2>&1
    if [[ $? -eq 0  ]];then
        `cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |\
                            grep -w -q "OFFLINE"` >/dev/null 2>&1
        if [[ $? -eq 0 ]];then
            sg_log 0 "WARNING: Both ASM failgroups of configured ASM diskgroup"\
                     "\"$dg_name\" in package \"$CURRENT_PKG\" are not in sync"
            exit_code=1
            if [[ -f $DG_STATUS_FILE ]];then
                sg_log 5 "Diskgroup status file \"$DG_STATUS_FILE\" found"
                sg_log 5 "Checking status of an online failgroup \"$fg_name\""\
                         "of diskgroup \"$dg_name\" in diskgroup status"\
                         "file"
                if [[ -z "${CLEAN_FG_IN_DG[$dg_index]}" ||\
                      "${CLEAN_FG_IN_DG[$dg_index]}" = "$FALSE" ]];then
                    CLEAN_FG_IN_DG[$dg_index]=$FALSE
                fi
            fi
        else 
            if [[ -f $DG_STATUS_FILE ]];then 
                sg_log 5 "Diskgroup status file \"$DG_STATUS_FILE\" found"
                sg_log 5 "Checking status of an online failgroup \"$fg_name\""\
                         "of diskgroup \"$dg_name\" in diskgroup status"\
                         "file"
                `cat $DG_STATUS_FILE 2>/dev/null |\
                         grep -q -w "$dg_name:$fg_name:OFFLINE:"` >/dev/null 2>&1
                if [[ $? -eq 0 ]];then 
                    if [[ -z "${CLEAN_FG_IN_DG[$dg_index]}" ||\
                          "${CLEAN_FG_IN_DG[$dg_index]}" = "$FALSE" ]];then 
                        CLEAN_FG_IN_DG[$dg_index]=$FALSE
                    fi 
                else 
                    CLEAN_FG_IN_DG[$dg_index]=$TRUE
                    sg_log 0 "INFO: Data on an online failgroup \"$fg_name\" of"\
                             "diskgroup \"$dg_name\" is clean"
                fi 
                exit_code=1
            fi 
        fi
    else
        exit_code=2
    fi
    cleanup_temp_cmd_log_file
    return $exit_code
}

###############################################################################
# This function caches DISKGROUP, FAILGROUP
# Internal Use only
#:
#
# Arguments:  diskgroup name 
#
#
# Returns:    1 for error, 0 for success 
#
###############################################################################
function cache_dg_fg_info
{
    sg_log 5 "Function cache_dg_fg_info"
    typeset dg_name=$1
    typeset -i dg_index=$2 
    typeset fg_list
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select d.FAILGROUP from v\\\$asm_diskgroup dg, 
            v\\\$asm_disk d where dg.group_number=d.group_number 
            AND dg.name='$dg_name';"
    result=\`echo "\$sqltext"| \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
    # beginingLineindex is the line number of a line contains
    # pattern like "-----------------------------" after that all
    # failgroup names will be listed for a diskgroup.
    beginingLineindex=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null|\
                      grep -n -E '^-+$'| cut -d':' -f1` >/dev/null 2>&1
    # terminalLineindex is the line number of a line which ends
    # with pattern "SQL> Disconnected from Oracle Database .."
    terminalLineindex=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null|\
                      grep -n -E '^SQL> Disconnected'|\
                      cut -d':' -f1` >/dev/null 2>&1
    # B/w beginingLineindex and terminalLineindex number all failgroups will be
    # listed.
    result=`expr $terminalLineindex - $beginingLineindex - 1` >/dev/null 2>&1
    fg_list=($(cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null| \
            grep -A $result -E '^-+$'| grep -v -E '^-+$'|sort -u))
    cleanup_temp_cmd_log_file
    if [[ -z "$fg_list" ]];then 
        sg_log 0  "INFO: ASM diskgroup \"$dg_name\" does not have any"\
                  "failgroups configured in package \"$CURRENT_PKG\""
        return 0        
    fi

    if [[ ${#fg_list[@]} -gt 2 ]];then 
        sg_log 0  "ERROR: More than two failgroups found for configured"\
                  "ASM diskgroup \"$dg_name\" for ASM mirroring in "\
                  "package \"$CURRENT_PKG\"" 
        return 1        
    fi
    NUM_OF_FG_IN_DG[$dg_index]=${#fg_list[@]}
    for fg in "${fg_list[@]}"
    do
        cache_fg_sync_info $fg $dg_name $dg_index >/dev/null 2>&1
        if [[ $? -eq 1 ]];then 
            if [[ -z "${IS_FG_SYNC_IN_DG[$dg_index]}" ]] ||\
               [[ "${IS_FG_SYNC_IN_DG[$dg_index]}" = "${FALSE}" ]];then
                IS_FG_SYNC_IN_DG[$dg_index]=$FALSE
            fi 
        elif [[ $? -eq 2 ]];then 
            sg_log 0  "ERROR: Unable to fetch ASM device status "\
                      "on failgroup \"$fg\" of configured "\
                      "diskgroup \"$dg_name\" for ASM mirroring in "\
                      "package \"$CURRENT_PKG\""
            return 1
        else
            IS_FG_SYNC_IN_DG[$dg_index]=$TRUE
        fi 
    done
    sg_log 5 "CLEAN_FG_IN_DG[$dg_index]: ${CLEAN_FG_IN_DG[$dg_index]}"
    sg_log 5 "IS_FG_SYNC_IN_DG[$dg_index]: ${IS_FG_SYNC_IN_DG[$dg_index]}"
    # Below function will ensure if DG is having single FG
    # then "IS_FG_SYNC_IN_DG" flag will be not set to TRUE or FALSE
    # so that RPO will not be checked for those DG. 
    check_dg_redundancy_type $dg_name >/dev/null 2>&1
    if [[ $? -ne 0 ]];then
        IS_FG_SYNC_IN_DG[$dg_index]=$NA
    fi 
    return 0
}
###############################################################################
# This function will show status of ASM disk, if disk status is online then
# will return "online" for all other status like "syncing"/"offline" will 
# return "offline".
# Internal Use only
#
#
# Arguments:  ASM Device Name
#
#
# Returns:    "0" [true ](if device is online) or
#             "1" [false](if device is offline)
#
###############################################################################
function get_asm_disk_status {
    typeset asm_dsk=$1
    typeset status
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select MODE_STATUS from v\\\$asm_disk where name='$asm_dsk';"
    result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > "${ORA_ASM_MIR_LOG_FILE}"
EOF
    cat "${ORA_ASM_MIR_LOG_FILE}" 2>/dev/null |grep -q -w -e "ONLINE"
    if [[ $? -eq 0 ]];then
        status=$ONLINE
    else
        status=$OFFLINE
    fi
    cleanup_temp_cmd_log_file
    return $status
}

###############################################################################
# This function will check if ASM disk is online.
# Internal Use only
#
#
# Arguments:  ASM Device Name
#
#
# Returns:    "0" [true ](if device is online) or 
#             "1" [false](if device is offline)
#
###############################################################################
function is_asm_disk_online {
    typeset asm_dsk=$1
    typeset ret_val
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select MODE_STATUS from v\\\$asm_disk where name='$asm_dsk';"
    result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > "${ORA_ASM_MIR_LOG_FILE}"
EOF
    cat "${ORA_ASM_MIR_LOG_FILE}" 2>/dev/null |grep -q -w -e "ONLINE" -e "SYNCING"
    if [[ $? -eq 0 ]];then
        ret_val=$TRUE
    else 
        ret_val=$FALSE
    fi 
    cleanup_temp_cmd_log_file
    return $ret_val
}

###############################################################################
# This function will bring back an ASM disk to online under ASM. 
# Internal Use only
#  
#
# Arguments:  DISKGROUP and ASM device name
#
#
# Returns:    none
#
###############################################################################
function bring_asm_disk_online 
{
    sg_log 5 "Function bring_asm_disk_online"
    typeset dg=$1
    typeset asm_dsk_name=$2 
    typeset raw_disk=$3
    typeset pkg_log_msg=""
    typeset status
    ERROR_MSG=""
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="ALTER DISKGROUP $dg ONLINE DISK $asm_dsk_name;"
    result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > "${ORA_ASM_MIR_LOG_FILE}"
EOF
    cat "${ORA_ASM_MIR_LOG_FILE}" 2>/dev/null |grep -q -w "Diskgroup altered."
    if [[ $? -eq 0 ]];then 
        sg_log 0 "Successfully made ASM disk \"$asm_dsk_name\" created on"\
                 "PV \"$raw_disk\" ONLINE or SYNCING"
        sg_log 0 "for the ASM diskgroup \"$dg\" configured in"\
                 "package $CURRENT_PKG"
        cleanup_temp_cmd_log_file
    else 
        error_msg=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |grep "ORA-"`
        cleanup_temp_cmd_log_file
        is_asm_disk_online $asm_dsk_name >/dev/null 2>&1
        status=$?
        if [[ "X$status" = "X$FALSE" ]];then
            pkg_log_msg="ERROR: Unable to make ASM disk \"$asm_dsk_name\""
            pkg_log_msg="$pkg_log_msg created on PV \"$raw_disk\"" 
            pkg_log_msg="$pkg_log_msg online for the ASM diskgroup \"$dg\"" 
            pkg_log_msg="$pkg_log_msg configured in package "
            pkg_log_msg="$pkg_log_msg $CURRENT_PKG due to "
            if [[ -n "$error_msg" ]];then 
                pkg_log_msg="$pkg_log_msg $error_msg"
            else  
                pkg_log_msg="$pkg_log_msg application error"
            fi 
            sg_log 0 $pkg_log_msg
        else 
            sg_log 0 "ASM disk \"$asm_dsk_name\" created on PV \"$raw_disk\""\
                     "is already in ONLINE or SYNCING mode"
            sg_log 0 "for the ASM diskgroup \"$dg\" configured in"\
                     "package $CURRENT_PKG"
        fi 
    fi
    ERROR_MSG=$pkg_log_msg
}

###############################################################################
# This function fetch DISKGROUP redundancy type,
# Internal Use only
#
#
# Arguments: RAW disk name
#
# Global var used:  ASM device path (/dev/oracleasm/disks/<DISK> or ORCL:<DISK>
#
# Returns:    0 if DISKGROUP redundancy type is "NORMAL"
#             1 if DISKGROUP redundancy type is other than "NORMAL"
#
###############################################################################
function check_dg_redundancy_type
{
    sg_log 5 "Function check_dg_redundancy_type"
    typeset dg_name=$1
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select TYPE from v\\\$asm_diskgroup
            where name='$dg_name';"
    result=\`echo "\$sqltext"| \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
    `cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null| grep -q -w "NORMAL"` >/dev/null 2>&1
    if [[ $? -ne 0 ]];then
        sg_log 0  "INFO: ASM diskgroup \"$dg_name\" will not be monitored as"\
                  "redundancy type is not \"NORMAL\""
        return 1
    fi
    cleanup_temp_cmd_log_file
    return 0
}

###############################################################################
# This function fetch DISKGROUP name from passed RAW disk and respective ASM 
# device name.
#
# Internal Use only
#  
# Arguments: RAW disk name
#
# Global Var: ASM Device Name
# 
# Global var sets:  DISKGROUP
#
# Returns:   0 (if dg found from asm device and is mirrored) otherwise 1
#
###############################################################################
function get_dg_from_asm_dev 
{
    sg_log 5 "Function get_dg_from_asm_dev"
    typeset raw_disk=$1
    typeset exit_code=0
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select dg.name from v\\\$asm_diskgroup dg, v\\\$asm_disk d 
            where dg.group_number=d.group_number AND 
            d.path='${ASM_DEV_PATH}';"
    result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
    ASM_DG=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |\
            grep -A 1 -E '^-+$'| grep -v -E '^-+$'` >/dev/null 2>&1
    if [[ -z $ASM_DG ]];then 
        sg_log 0  "$TYPE_OF_MSG: Unable to find corresponding ASM diskgroup name"\
                  "for disk \"$raw_disk\" configured in package $CURRENT_PKG"
        exit_code=1
    else 
        check_dg_redundancy_type $ASM_DG >/dev/null 2>&1
        if [[ $? -ne 0 ]];then 
            exit_code=1
        fi 
    fi
    cleanup_temp_cmd_log_file
    return $exit_code
}

###############################################################################
# This function fetch FAILGROUP name from global variable ASM device path
# Internal Use only
#
# Arguments:  NONE
#
# Global var used:  ASM device path (/dev/oracleasm/disks/<DISK> or ORCL:<DISK>
#
# Global var sets:  FAILGROUP
#
# Returns:   NONE
#
###############################################################################
function get_fg_from_asm_dev 
{
    sg_log 5 "Function get_fg_from_asm_dev"
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select FAILGROUP from v\\\$asm_disk 
            where path='${ASM_DEV_PATH}';"
    result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
    ASM_FG=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |\
            grep -A 1 -E '^-+$'| grep -v -E '^-+$'` >/dev/null 2>&1
    cleanup_temp_cmd_log_file
}

###############################################################################
# This function fetch ASM disk name from passed RAW disk name and AFD device 
# path
# Internal Use only
#
#
# Arguments:  RAW disk name
#
# Global var sets:  ASM disk name
#
# Returns: NONE
#
###############################################################################

function get_asm_dsk_name_from_afd_dev_path
{
    sg_log 5 "Function get_asm_dsk_name_from_afd_dev_path"
    typeset raw_disk=$1
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select NAME from v\\\$asm_disk
            where path='${ASM_DEV_PATH}';"
    result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
    ASM_DSK_NAME=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |\
                 grep -A 1 -E '^-+$'| grep -v -E '^-+$'` >/dev/null 2>&1
    if [[ -z $ASM_DSK_NAME ]];then
        sg_log 0  "$TYPE_OF_MSG: Unable to find corresponding ASM disk"\
                  "name of ASM disk path \"${ASM_DEV_PATH}\" for"\
                  "disk \"$raw_disk\" in package \"$CURRENT_PKG\""
    fi
    cleanup_temp_cmd_log_file
}

###############################################################################
# This function fetch ASM disk name from passed RAW disk name and ASM Lib
# device name
# Internal Use only
#  
#
# Arguments:  RAW disk name
#
# Global var sets:  ASM disk name
#
# Returns:  NONE
# 
###############################################################################
function get_asm_dsk_name_from_asm_lib_dev_path 
{
    sg_log 5 "Function get_asm_dsk_name_from_asm_lib_dev_path"
    typeset raw_disk=$1
    ASM_DEV_PATH="${ORACLEASM_LIB_DISKS}/$ASM_DEV"
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select NAME from v\\\$asm_disk 
            where path='${ASM_DEV_PATH}';"
    result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
    ASM_DSK_NAME=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |\
                 grep -A 1 -E '^-+$'| grep -v -E '^-+$'` >/dev/null 2>&1
    if [[ -z $ASM_DSK_NAME ]];then 
        su - $ORA_ADMIN -c "/bin/bash" <<EOF
        export ORACLE_HOME=$ORA_HOME
        export ORACLE_SID=$ORA_SID
        sqltext="select NAME from v\\\$asm_disk
                 where path='ORCL:$ASM_DEV';"
        result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
        echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
        ASM_DSK_NAME=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |\
                 grep -A 1 -E '^-+$'| grep -v -E '^-+$'` >/dev/null 2>&1
        if [[ -z $ASM_DSK_NAME ]];then 
             sg_log 0  "$TYPE_OF_MSG: Unable to find corresponding"\
                       "ASM disk name of ASM disk path \"${ASM_DEV_PATH}\""\
                       "for disk \"$raw_disk\" in package \"$CURRENT_PKG\"" 
        else 
            ASM_DEV_PATH="ORCL:$ASM_DEV"
        fi
    fi
    cleanup_temp_cmd_log_file
}

###############################################################################
# This function will return all raw disks of a DM device
# Internal Use only
#
#
# Arguments:  DM device name
#
#
# Returns:    ARRAY OF RAW DISKs
#
###############################################################################
function get_raw_disk_from_dm
{
    typeset dm_dev=/dev/${1}
    test -f /etc/redhat-release && \
    if /bin/grep "Maipo" /etc/redhat-release >/dev/null 2>&1
    then
        OS_Version="RH7"
    fi
    part_number=`$UDEV_PROP $dm_dev 2>/dev/null |grep -w "DM_UUID" |\
                awk -F= '{print $NF}'|awk -F "-" '{print $1}'|tr -d 'part'`
    ## For Redhat 7, run MPATH_LIST over UUID of parent device ##
    if [ "$OS_Version" = "RH7" ]
    then
        dm_dev_uuid=`$DM_DETAILS $dm_dev | grep UUID | awk -F- '{print $NF}'`
        DM_LIST=$DM_LIST" "$($MPATH_LIST $dm_dev_uuid | grep sd |\
                grep active | sed 's/|//g' |\
                awk '{print $3}') > /dev/null 2>&1
    else
        DM_LIST=$DM_LIST" "$($MPATH_LIST $dm_dev | grep sd | grep active |\
                sed 's/|//g' | awk '{print $3}') > /dev/null 2>&1
    fi
    for DEV_PATH in $DM_LIST
    do
        DEV_PATH="/dev/${DEV_PATH}$part_number"
        DM_DEV_LIST=$DM_DEV_LIST" "$DEV_PATH
    done
    echo ${DM_DEV_LIST[@]}
}

###############################################################################
# This function fetch ASM Filter driver device name from ASM device label 
# Internal Use only
#
# Arguments:  NONE
#
# Global var sets:  ASM device path (/dev/oracleafd/disks/<DISK> or AFD:<DISK>)
#                   ASM device name (/dev/oracleafd/disks/<DISK> or AFD:<DISK>)
#
# Returns:    NONE
#
###############################################################################
function get_afd_dev_from_asm_dev_label
{
    sg_log 5 "Function get_path_from_asm_label"
    dev_label=$1
    recreate_tmp_cmd_log_file
    su - $ORA_ADMIN -c "/bin/bash" <<EOF
    export ORACLE_HOME=$ORA_HOME
    export ORACLE_SID=$ORA_SID
    sqltext="select PATH from v\\\$asm_disk
            where label='${dev_label}';"
    result=\`echo "\$sqltext" | \$ORACLE_HOME/bin/sqlplus '/ as sysasm' \`
    echo "\$result" > ${ORA_ASM_MIR_LOG_FILE}
EOF
    ASM_DEV=`cat ${ORA_ASM_MIR_LOG_FILE} 2>/dev/null |\
            grep -A 1 -E '^-+$'| grep -v -E '^-+$'` >/dev/null 2>&1
    if [[ -n $ASM_DEV ]];then 
        ASM_DEV_PATH=${ASM_DEV}
    fi 
    cleanup_temp_cmd_log_file
}

###############################################################################
# This function fetch ASM device name (created using AFD) from passed RAW disk
# name.
# Internal Use only
#
# Arguments:  NONE
#
# Global var set:  ASM device path (/dev/oracleafd/disks/<DISK> or AFD:<DISK>)
#
# Returns:    NONE
#
###############################################################################
function get_asm_fltr_drivr_dev_from_raw_disk
{
    sg_log 5 "Function get_asm_fltr_drivr_dev_from_raw_disk"
    typeset raw_disk=$($UDEV_INFO $1 2>/dev/null)
    typeset dev_label=""
    export ORACLE_HOME=$ORA_HOME
    if [[ -n $raw_disk ]]
    then
        echo $raw_disk |grep -q "dm-"
        if [[ $? -eq 0 ]];then
            tmp=($(get_raw_disk_from_dm $raw_disk))
            raw_disk=${tmp[@]}
        fi
        # Appending udev device name also to raw_disk array 
        # to find device label from all underlying raw disks
        # and udev device name.
        raw_disk[${#raw_disk[@]} + 1]=$1
        for i in ${raw_disk[@]};
        do
            dev_label=$($ORACLE_HOME/bin/asmcmd afd_lsdsk 2>/dev/null |\
                        grep "$i"|awk '{print $1}') >/dev/null 2>&1
            if [[ -n $dev_label ]];then
                break
            fi
        done
        if [[ -n $dev_label ]];then
            get_afd_dev_from_asm_dev_label $dev_label >/dev/null 2>&1
        fi 
    fi
}

###############################################################################
# This function fetch ASM device name (created using ASM Lib) from passed 
# RAW disk name.
# Internal Use only
#  
#
# Arguments:  RAW disk name
#
# Global var set:  ASM device name
#
# Returns:   NONE
#
###############################################################################
function get_asm_lib_dev_from_raw_disk
{
    sg_log 5 "Function get_asm_lib_dev_from_raw_disk"
    typeset raw_disk=$($UDEV_INFO $1 2>/dev/null)
    if [[ -n $raw_disk ]]
    then
        echo $raw_disk |grep -q "dm-"
        if [[ $? -eq 0 ]];then   
            tmp=($(get_raw_disk_from_dm $raw_disk))
            raw_disk=${tmp[@]}
        fi 
        for i in $raw_disk;
        do 
           ASM_DEV=$(ls -l /dev/oracleasm/disks/ 2>/dev/null | grep -v "^total" |\
                     awk '{print $5" "$6" "$NF }'|sed 's/,//g' |\
                 while read v_minor v_major v_asmdisk; \
                 do v_device=`ls -la /dev | grep " $v_minor, *$v_major " |\
                     awk '{print $10}'`; echo "$v_asmdisk /dev/$v_device";\
                 done | grep $i|awk '{print $1}')
            if [[ -n $ASM_DEV ]];then
                break  
            fi 
        done
    fi 
}

###############################################################################
# This function fetch ASM device name from passed RAW disk name.
# NOTE: ASM device can be created using either Oracle ASM Lib or 
#       Oracle ASM filter driver (AFD)
# Internal Use only
#
# Arguments:  RAW disk name
#
# Global var set:  ASM device name
#
# Returns:    NONE
#
###############################################################################
function get_asm_dev_from_raw_disk
{
    sg_log 5 "Function get_asm_dev_from_raw_disk"     
    typeset raw_disk=$1
    if [[ "$ORA_ASM_DRIVER_TYPE" = "ALIB" ]];then 
        # Find if ASM DEVICE created using Oracle ASM Lib
        get_asm_lib_dev_from_raw_disk $raw_disk >/dev/null 2>&1
    elif [[ "$ORA_ASM_DRIVER_TYPE" = "AFD" ]];then
        # Find if ASM DEVICE created using Oracle ASM filter driver
        get_asm_fltr_drivr_dev_from_raw_disk $raw_disk >/dev/null 2>&1
    fi 
    # If failed to find ASM_DEV using both Oracle ASM Lib or ASM filter driver
    # way then show warning or error for that raw disk
    if [[ -z $ASM_DEV ]];then 
       sg_log 0  "$TYPE_OF_MSG: Unable to find corresponding ASM device"\
                 "name for disk \"$raw_disk\" configured in package $CURRENT_PKG"
    fi
}
###############################################################################
# This function fetch ASM device info like DISKGROUP, failgroup, ASM device name
# and ASM disk name from passed RAW disk name.
# Internal Use only
#  
#
# Arguments:  RAW disk name
#
#
# Returns:    ASM DEVICE INFO FOR AN ASM PV
#
###############################################################################
function populate_asm_dev_info
{
    sg_log 5 "Function populate_asm_dev_info"
    typeset raw_disk=$1
    typeset -i excluded_asm_pv_for_monitor=$FALSE
    typeset -i ret_val=0
    typeset -i is_dg_mirrored=$FALSE
    ASM_DG=""
    ASM_FG=""
    ASM_DSK_NAME=""
    ASM_DEV_PATH=""
    ASM_DEV=""
    # get_asm_dev_from_raw_disk will populate both ASM_DEV and ASM_DEV_PATH
    # if ASM device has been created using AFD.
    # get_asm_dev_from_raw_disk will populate only ASM_DEV, if ASM device 
    # has been created using Oracle ASM Lib.
    get_asm_dev_from_raw_disk $raw_disk
    if [[ -n "$ASM_DEV" ]];then 
        if [[ -n "$ASM_DEV_PATH" ]];then 
            # As Global var ASM_DEV_PATH has been set already that means 
            # ASM device has been created using AFD.
            get_asm_dsk_name_from_afd_dev_path $raw_disk >/dev/null 2>&1
        else 
            # As Global var ASM_DEV_PATH has not been set that means 
            # ASM device has been created using ASM Lib.
            get_asm_dsk_name_from_asm_lib_dev_path $raw_disk >/dev/null 2>&1
        fi 
        if [[ -n $ASM_DSK_NAME ]];then 
            get_dg_from_asm_dev $raw_disk >/dev/null 2>&1
            # Variable "is_dg_mirrored" will be set to TRUE if dg is mirrored
            # otherwise FALSE
            is_dg_mirrored=$?
            if [[ -n $ASM_DG ]];then
                if [[ $is_dg_mirrored -eq $TRUE ]];then
                    get_fg_from_asm_dev >/dev/null 2>&1
                    if [[ -z $ASM_FG ]];then
                        excluded_asm_pv_for_monitor=$TRUE
                    fi
                fi
            else 
                excluded_asm_pv_for_monitor=$TRUE
            fi 
        else 
            excluded_asm_pv_for_monitor=$TRUE
        fi 
    else 
        # Either ASM device is not accessible or has not been created
        # using AFD or ASM Lib.
        excluded_asm_pv_for_monitor=$TRUE
      
    fi
    if [[ ! -f $ORA_ASM_MIR_CONF_FILE ]];then 
        echo "$ASM_DG:$ASM_FG:$ASM_DSK_NAME:$raw_disk" >> "${ORA_ASM_MIR_CONF_FILE}.tmp" 
    fi 
    if [[ $excluded_asm_pv_for_monitor -eq $TRUE ]];then 
        ret_val=1
    fi 
    if [[ $ret_val -eq 1 ]] && [[ "${TYPE_OF_MSG}" = "ERROR" ]];then 
         sg_log 0  "ERROR: ASM PV \"$raw_disk\" will not be monitored as"\
                   "does not contain either ASM diskgroup or failgroup information"
    fi 
    return $ret_val
}

###############################################################################
# This function fetch ASM device info for all PVs listed in ASM_PV_LIST.
# Internal Use only
#
# Arguments:  RAW disk name 
#
#
# Returns:    ASM DEVICE INFO FOR ALL ASM PVs
#
###############################################################################
function populate_asm_dev_list_info
{
    # Possible ASM_PV device can be as mentioned below:
    # 1. ASM_PV having mapping as valid ASM device also having DG & FG configured.
    # 2. ASM_PV having mapping as valid ASM device also having DG & FG configured
    #    ,but having hardware issue from startup of package.
    # 3. ASM_PV can have mapping as valid ASM device also having DG & FG 
    #    configured, but user set ASM device as "offline" from startup of package.
    # 4. ASM_PV do not have any mapping to ASM device (Does not contain DG & 
    #    FG as well)
    # 5. ASM_PV do not have any mapping to ASM device (Does not contain DG & 
    #    FG as well) and and having hardware issue from startup of package
    # 6. ASM_PV having mapping to ASM device but does not configured for any DG
    #    & FG.
    # 7. ASM_PV having mapping to ASM device but does not configured for any DG
    #    & FG and also having hardware issue from startup of package.
    # Note: Only ASM_PV of type "1" will be monitored.
    typeset op=$1
    typeset -i index=0
    typeset ISSUE_ON_FIRST_TIME_STARTUP_OF_PKG=$FALSE
    sg_log 5 "Function populate_asm_dev_list_info"
    #If package is coming up on cluster very first time then for any 
    #issues with ASM device name, ASM disk, diskgroup or failgroup
    #tag all messages with error flag otherwise with warning flag.
    if [[ ! -f $DG_STATUS_FILE ]];then
        TYPE_OF_MSG="ERROR"
    fi 
    if [[ -f "${ORA_ASM_MIR_CONF_FILE}.tmp" ]];then 
        rm -rf "${ORA_ASM_MIR_CONF_FILE}.tmp"
    fi 
     
    for ora_asm_pv in "${ORACLE_ASM_PV[@]}"
    do
        populate_asm_dev_info $ora_asm_pv >/dev/null 2>&1
        if [[ $? -eq 1 ]] && [[ "${TYPE_OF_MSG}" = "ERROR" ]];then 
            ISSUE_ON_FIRST_TIME_STARTUP_OF_PKG=$TRUE
        fi 
        index=$(($index+1))        
    done
    if [[ "$ISSUE_ON_FIRST_TIME_STARTUP_OF_PKG" = "$TRUE" ]];then
        sg_log 0 "Rectify all errors before start the package again"
        return 1
    fi 
    #Distribute ASM mirror configutration file to all configured nodes
    #on below cases:
    # 1. First time startup of package on cluster, when there will
    #    be no ASM mirror configuration file.
    # 2. Online reconfiguration of package when there is change 
    #    in "ASM_PV" parameter.
    if [[ "${op}" = "reconfigure_start" ]] ||\
       [[ "${op}" = "start" && ! -f ${ORA_ASM_MIR_CONF_FILE} ]];then
        sg_log 0 "Copying Oracle ASM mirror configuration"\
                 "file \"${ORA_ASM_MIR_CONF_FILE}\" to all package owner nodes."
        copy_file "${ORA_ASM_MIR_CONF_FILE}.tmp" "${ORA_ASM_MIR_CONF_FILE}"
        if [[ $? -ne 0 ]];then
            sg_log 0 "ERROR: Failed to copy Oracle ASM mirror configuration"\
                     "file \"$ORA_ASM_MIR_CONF_FILE\" to all package owner nodes."
            return 1
        fi
    fi 
    #Remove the temporary ASM mirror configutration file 
    if [[ -f "${ORA_ASM_MIR_CONF_FILE}.tmp" ]];then 
       rm -rf "${ORA_ASM_MIR_CONF_FILE}.tmp"
    fi 
    return 0   
}

#######################################################################
#
# function ora_mirror_get_monitor_service() gets the name of Oracle ASM 
# mirror monitoring service.
#
#######################################################################

function ora_mirror_get_monitor_service
{
    sg_log 5 "Function ora_mirror_get_monitor_service"
    if [[ ${#SG_SERVICE_NAME[@]} -ne 0 ]]; then
        index=0
        for serv in ${SG_SERVICE_NAME[@]}
        do
            FOUND=`echo "${SG_SERVICE_CMD[$index]}" |\
                  grep -w 'asm_mirror.mon'` > /dev/null 2>&1
            if [[ "$FOUND" != "" ]]
            then
                 monitoring_service_name=$serv
                 monitoring_service_cmd=${SG_SERVICE_CMD[$index]}
                 sg_log 5 "Found monitoring service "\
                          "${monitoring_service_name}"
                 return
            fi
            index=$(($index+1))
        done
    fi
}

#######################################################################
#
# function check_asm_lib_driver_state checks Oracle ASM lib
# driver state, if ASM Lib is configured properly it will return true
# otherwise false
#
#######################################################################
function check_asm_lib_driver_state 
{ 
    sg_log 5 "Function check_asm_lib_driver_state"
    typeset is_drv_loaded=$FALSE
    $ORACLEASM status 2>/dev/null|grep -i "loaded" |grep -q -i "yes"
    if [[ $? -eq 0 ]];then
        sg_log 5 "INFO: Oracle ASM Lib driver is configured"
        is_drv_loaded=$TRUE
    fi 
    return $is_drv_loaded
}

#######################################################################
#
# function check_asm_lib_driver_access checks for Oracle ASM Lib driver
# related settings
#
#######################################################################
function check_asm_lib_driver_access
{
    sg_log 5 "Function check_asm_lib_driver_access"
    typeset -i exit_code=0
    typeset oracleasm_conf_params
    oracleasm_conf_params=($($ORACLEASM configure 2>/dev/null))
    if [[ -n "${oracleasm_conf_params[@]}" ]]
    then 
        for para in ${oracleasm_conf_params[@]}
        do
            `echo $para|grep -q "_UID"` >/dev/null 2>&1
            if [[ $? -eq 0 ]]
            then 
                uid=`echo $para|awk -F "=" '{print $NF}'`
            else 
                `echo $para|grep -q "_GID"` >/dev/null 2>&1
                if [[ $? -eq 0 ]]
                then 
                    gid=`echo $para|awk -F "=" '{print $NF}'`
                else 
                    `echo $para|grep -q "_SCANBOOT"` >/dev/null 2>&1
                    if [[ $? -eq 0 ]]
                    then 
                        set_scan_boot=`echo $para|\
                            awk -F "=" '{print $NF}'` >/dev/null 2>&1
                    fi 
                fi 
            fi 
        done 
        if [[ -z "$uid" ]] 
        then 
            sg_log 0 "ERROR: Default user for the Oracle ASM "\
                     "library driver is not set"
            exit_code=1
        fi 
        if [[ -z "$gid" ]] 
        then 
            sg_log 0 "ERROR: Default group for the Oracle ASM "\
                     "library driver is not set"
            exit_code=1
        fi 
        if [[ $set_scan_boot != "true" ]] 
        then 
            sg_log 0 "ERROR: Fix permissions of Oracle ASM disks "\
                     "on boot is not set to \"true\""
            exit_code=1
        fi 
        return $exit_code
    else 
        sg_log 0 "ERROR: Unable to get information for the Oracle ASM "\
                 "library driver configuration on \"$HOST\""
        return 1 
    fi 
}

#######################################################################
#
# function check_asm_fltr_driver_state checks Oracle ASM filter 
# driver state, if AFD is configured properly it will return true
# otherwise false
#
#######################################################################
function check_asm_fltr_driver_state
{
    sg_log 5 "Function check_asm_fltr_driver_state"
    typeset is_drv_loaded=$FALSE
    export ORACLE_HOME=$ORA_HOME
    $ORACLE_HOME/bin/asmcmd afd_state 2>&1 |grep -q "LOADED"
    if [[ $? -eq 0 ]];then
        sg_log 5 "INFO: Oracle ASM filter driver is configured"
        is_drv_loaded=$TRUE
    fi 
    return $is_drv_loaded
}

#######################################################################
#
# function activate_pr_on_raw_disk sets the PR keys on recovered ASM 
# PVs after failure
#
#######################################################################
function activate_pr_on_raw_disk
{
    typeset DEVICE=$1
    typeset -i num_vmdk=0
    sg_log 5 "Function activate_pr_on_raw_disk"
    # If the node is a Physical Machine, PR activations must be
    # done. But if this is a Virtual Machine, then PR activations
    # must be done if the package failover happened ungracefully
    # from a Physical Machine to
    # a VM.
    echo "$cmviewcl_output" | grep -w "node:$HOST|\
         virt_tech=vmware" >> /dev/null 2>&1
    if [ $? -eq 0 ]; then
    # If there are VMFS files specified, skip PR activities
    sg_check_module_in_pkg $VMFS_MODULE
    if [ $? -eq 0 ]; then
          num_vmdk=${#SG_VMDK_FILE_NAME[*]}
          if [[ $num_vmdk -ge 1 ]] && ! sg_check_if_pr_module_required
          then
              return 0
          fi
       fi
    fi

    if [ "$PKG_PR_MODE" = "pr_enabled" ]
    then
        ${SGCONF}/scripts/sg/pr_util.sh ${DEVICE} "activate"
        if [ $? -ne 0 ]
        then
            sg_log 0 "Activating PR failed on "${DEVICE}
            if sg_check_if_it_is_reconfiguration
            then
                sg_log 999 "ERROR: Activating PR failed on "${DEVICE}
                sg_set_reconfig_failure
                to_exit=1
            else
                exit 1
            fi
        fi
    fi
    return 0
}

#######################################################################
#
# This function is to copy a file to all nodes configured for a package
#
#######################################################################
function copy_file
{
    sg_log 5 "Function copy_file"
    typeset source_file=$1
    typeset dest_file=$2
    typeset node_list="$(cmviewcl -f line -v -s config -p $CURRENT_PKG |\
                         grep "node:.*|name=" | cut -f2 -d'=')"
    typeset -i ret_val=0
    for node in ${node_list}
    do
        cmcp $source_file $node:$dest_file >/dev/null 2>&1
        if (( $? != 0 )); then
            sg_log 0 "ERROR: Failed to send file "\
                     "\"$(basename $dest_file)\" to Node \"$node\""
            ret_val=1
        fi
    done
    return $ret_val
}

#######################################################################
#
# This function is to restores ASM mirror conf file and disk group 
# status to all nodes configured for a package
#
#######################################################################
function restores_conf_status_file
{
    typeset -i exit_code=1
    sg_log 5 "Function restores_conf_status_file"
    if [[ -f "${ORA_ASM_MIR_CONF_FILE}.bk" ]];then 
        mv "${ORA_ASM_MIR_CONF_FILE}.bk" "${ORA_ASM_MIR_CONF_FILE}"
        copy_file "${ORA_ASM_MIR_CONF_FILE}" "${ORA_ASM_MIR_CONF_FILE}"
        if [[ $? -ne 0 ]];then
            sg_log 0 "ERROR: Failed to copy Oracle ASM mirror"\
                     "configuration file \"$ORA_ASM_MIR_CONF_FILE\" to"\
                     "all package owner nodes."
            exit_code=1
        else 
            exit_code=0
        fi
    fi 
    if [[ -f "${DG_STATUS_FILE}.bk" ]];then 
        mv "${DG_STATUS_FILE}.bk" "${DG_STATUS_FILE}"
        copy_file "${DG_STATUS_FILE}" "${DG_STATUS_FILE}"
        if [[ $? -ne 0 ]];then
            sg_log 0 "ERROR: Failed to copy Oracle disk group status"\
                     "file \"$DG_STATUS_FILE\" to all package owner nodes."
            exit_code=1
        else
            if [[ $exit_code -eq 0 ]];then
                exit_code=0
            fi 
        fi
    fi 
    return $exit_code
}

#######################################################################
#
# This function is to take a backup copy of ASM mirror conf file and 
# disk group status then remove them from all nodes configured for a 
# package.
#
#######################################################################
function cleanup_conf_status_file 
{
    sg_log 5 "Function cleanup_conf_status_file"
    if [[ -f "${ORA_ASM_MIR_CONF_FILE}" ]];then
        sg_log 5 "INFO: Removing Oracle ASM mirror configuration file"
        mv "${ORA_ASM_MIR_CONF_FILE}" "${ORA_ASM_MIR_CONF_FILE}.bk"
    fi
    if [[ -f "${DG_STATUS_FILE}" ]];then
        sg_log 5 "INFO: Removing Oracle diskgroup status file"
        mv "${DG_STATUS_FILE}" "${DG_STATUS_FILE}.bk"
    fi
}

#######################################################################
#
# This function is to remove backup copy of ASM mirror conf file and 
# disk group status from all nodes configured for a package.
#
#######################################################################
function cleanup_backup_conf_status_file
{
    if [[ -f "${ORA_ASM_MIR_CONF_FILE}.bk" ]];then
        sg_log 5 "INFO: Removing old backup-ed Oracle ASM mirror"\
                 "configuration file"
        rm -rf "${ORA_ASM_MIR_CONF_FILE}.bk"
    fi
    if [[ -f "${DG_STATUS_FILE}.bk" ]];then
        sg_log 5 "INFO: Removing old backup-ed Oracle diskgroup"\
                 "status file"
        rm -rf "${DG_STATUS_FILE}.bk"
    fi
}
