#!/bin/bash
# (c) Copyright 2015 Hewlett Packard Development Company, L.P.
# Version A.12.00.40
################################################################################
# Check the status of Oracle database
################################################################################
# This script connects to the Oracle database and gets its status.  If there is
# an error in getting the status of the database, then it returns with an error
# code 1.  If it succeeds in collecting the status, then it returns 0.  The 
# database is considered as 'Not-hung' if its status is any one of the 
# following:
#	STARTED
#	MOUNTED
#	OPEN
#	OPEN MOUNTED
#
# Since the database is not considered hung if the status command returns
# successfully, this script considers the database as hung only if the 
# status query returns an error.
#
# Returns:
#	0 - If the status query is successful
#	1 - If the status query returns an error
#
# REVISION HISTORY:
#     Version       mm/dd/yyyy   Changes
#     A.05.01       04/30/2012   1) Added Code to support database hang detection.
#     A.12.00.40    08/31/2015   1) Fix for QXCR1001430463 
#################################################################################

function db_get_status {
		
		$CMD ${ORACLE_ADMIN} -c "/bin/bash" <<STAT >/dev/null 2>&1
		exit_code=0;
		export ORACLE_HOME=${ORACLE_HOME}
		export ORACLE_SID=${SID_NAME}
                # On HP-UX a read statement is used to get the query result to a variable.
                # On Linux read statement is failing inside "su" hence the following code is used. 
                sqltext="SELECT STATUS FROM V\\\$INSTANCE;"
                stat=\`echo "\$sqltext" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'|grep "ORA-" \`	
		if [[ "\$stat" != "" ]]; then	
			exit_code=1
                        echo "$(date '+%b %e %X') - Node \"${HOST}\" ERROR: ORA-error during hang monitoring: \$stat" > /tmp/ora_error_${SID_NAME}.txt
		fi
		exit \$exit_code;
STAT
		exit_status=$?
                if [ $exit_status -eq 1 ]; then
                        if [[ -f /tmp/ora_error_${SID_NAME}.txt ]] ; then
                                cat /tmp/ora_error_${SID_NAME}.txt
                        else
                                echo "$(date '+%b %e %X') - Node \"${HOST}\" ERROR: ORA-error during hang monitoring."
                        fi
                fi

		return $exit_status;
}


# MAIN
db_get_status
return $?

