#!/bin/bash
##############################################################################
# (c) Copyright 2016 Hewlett-Packard Enterprise Development LP.
#
##############################################################################
# haoracle_utils.sh : HP Serviceguard toolkit for Oracle - Utility script. 
# Version A.12.00.50

##############################
# Source SG utility functions.
##############################


if [[ -z "$SG_UTILS" ]]; then
    SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
fi

if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        $ECHO "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    $ECHO "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi

###################################################################
# Get the environment for this package through utility function
# sg_source_pkg_env().
###################################################################

export PATH=$PATH:$SGSBIN

if [[ ! -f $2 ]] ; then
        sg_source_pkg_env $1
else
        sg_source_pkg_env $*
fi
CMD="su -"
#############################
# check_if_pdb_exists function
#############################
function check_if_pdb_exists
{
pdb=`echo $1|tr '[[:lower:]]' '[[:upper:]]'`
result=$( $CMD ${ORACLE_ADMIN} -s /bin/bash <<EOF
     export ORACLE_HOME=${ORACLE_HOME}
     export ORACLE_SID=${SID_NAME}
    echo "select concat ('PDBNAME:',name) as A from v\\\$pdbs where name = '$pdb';" | $ORACLE_HOME/bin/sqlplus '/' as sysdba
EOF
)
    echo "$result" | grep -i -w  "PDBNAME:$pdb"  >> /dev/null
     if [[ $? -ne 0 ]]; then
        return 1
     fi

}

##############################################################################
# Function check_oracle_version
#
# This function checks the version of Oracle DB using the opatch utility
# located in the ORACLE_HOME/bin directory.  If that does not succeed, then
# for ASM instance, it will try to find the oracle version using opatch
# utility from ASM_HOME/bin directory. If that also fails, then it will
# retry using the same method two more times before exiting. If it is still
# not able to find the Oracle version, then it will try to find the Oracle
# version using the Oracle's local inventory located at
# ORACLE_HOME/inventory/ContentsXML/comps.xml.
#
# If Oracle database version is 12cR1, it sets the ORACLE_VERSION
# paramter to 12cR1. If Oracle Database version is 11gR2 it sets the
# ORACLE_VERSION parameter to 11gR2.If Oracle Database version is 11gR1,
# then it sets the ORACLE_VERSION parameter to 11gR1.  Else to 10gR.
############################################################################
function check_oracle_version
{
        local oracle_11gr1="11.1.0.1.0"
        local oracle_11gr2="11.2.0.1.0"
        local oracle_12cr1="12.1.0.1.0"
        retry=2

        while true
        do
                if [[ "$INSTANCE_TYPE" = "database" ]]; then
                db_version=`/bin/su - $ORACLE_ADMIN -c '/bin/bash' <<EOF 2>/dev/null
                export ORACLE_HOME=$ORACLE_HOME
                $ORACLE_HOME/OPatch/opatch lsinventory -oh $ORACLE_HOME | grep -E "^Oracle Database"
EOF`
                fi
                #For ASM instance only, try to check the version of CRS if the above command fails.
                        if [[ "$INSTANCE_TYPE" = "ASM" ]]; then
                                db_version=`/bin/su - $ORACLE_ADMIN -c '/bin/bash' <<EOF1 2>/dev/null
                                export ORACLE_HOME=$ORACLE_HOME
                                $ORACLE_HOME/OPatch/opatch lsinventory -oh $ORACLE_HOME | grep -e "^Oracle Grid Infrastructure" -e "Oracle Clusterware"
EOF1`
                        fi

                if [[ -z "$db_version" ]]
                then
                        if [[ $retry -gt 0 ]]
                        then
                                random_delay=`expr '(' $RANDOM '*' 10 / 32767 ')' + 1`
                                sleep $random_delay # try again after a while
                        else
                                # Did not find DB version after multiple retries
                                break
                        fi
                else
                        #Found DB version using opatch
                        break;
                fi
                (( retry = $retry - 1 ))
        done

        for ver in `echo $db_version | grep "Oracle Database" | awk '{print $NF}'`
        do
                up_db_version=$ver
        done
        db_version=$up_db_version
        # Could not find DB version using the standard opatch utility.
        # Making a final attempt to find the oracle version using the local inventory.
        #   This method may not succeed all the time.
        if [[ -z "$db_version" ]]; then
            db_version=`/bin/sed -e '/^<COMP NAME="oracle.server" VER="/p;D' $ORACLE_HOME/inventory/ContentsXML/comps.xml | awk -F \" 'END {print $4}'`
                if [ $? -ne 0 -o -z "$db_version" ]; then
                        if [[ "$INSTANCE_TYPE" = "ASM" ]]; then
                           db_version=`/bin/sed -e '/^<COMP NAME="oracle.crs" VER="/p;D' $ORACLE_HOME/inventory/ContentsXML/comps.xml | awk -F \" ' END {print $4}'`
                        fi
                fi
        fi

        if [[ -z "$db_version" ]]
        then
                echo "ERROR: Unable to get Oracle Database version."
                exit 1
        fi
        ret=`echo "" | awk '{ if ('$db_version' >= '$oracle_12cr1'){
                        print "12cR1"
                } else if ('$db_version' >= '$oracle_11gr2'){
                        print "11gR2"
                } else if ('$db_version' >= '$oracle_11gr1'){
                        print "11gR1"
                } else {
                        print "10gR"
                }
         }'`

       ORACLE_VERSION=$ret
       echo "$ORACLE_VERSION"

}


###############################################################################
# Function: pdb_state_change
# This function is responsible for starting/stopping the PDBs associated with CDB.
# Arguments:
# open - will open all PDB in READ WRITE MODE except PDB$SEED.
# close - will close all PDB before shutting down CDB. If USER_SHUTDOWN_MODE
#         attribute is set  to "immediate" then PDB will be closed with
#         immediate option.
###############################################################################
function pdb_state_change
{
state="$1"
$CMD ${ORACLE_ADMIN} -c "/bin/bash"  <<EOF >/dev/null 2>&1 
export ORACLE_HOME=${ORACLE_HOME}
export ORACLE_SID=${SID_NAME}
if [[ "$INSTANCE_TYPE" = "database" ]] && [[ "$START_MODE" = "open" ]]; then
   if [[ "${state}" = "close" && "${USER_SHUTDOWN_MODE}" = "immediate" ]]; then
      echo "alter pluggable database all ${state} immediate;" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
   else
      echo "alter pluggable database all ${state};" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
   fi
   if [[ \$? -ne 0 ]]; then
      echo "WARNING: Unable to ${state} one or more pluggable database."
   fi
else
   exit 0;
fi
exit 0;
EOF
}

####################################################################################
# Function: determine_db_type
# Description: This function is responsible for determining the type of the database.
# Prerequisite: Database instance should be up and running to determine the
# type of database.
# Returns:
# YES - If the database packaged is CDB.
# NO  - If the database packaged is non CDB.
#####################################################################################

function determine_db_type
{
#set -x
result=$( $CMD $ORACLE_ADMIN -c "/bin/bash" <<EOF
export ORACLE_HOME=${ORACLE_HOME}
export ORACLE_SID=${SID_NAME}
echo "select concat('DBTYPE:',CDB) A from v\\\$DATABASE;" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
exit
EOF
)
CDB=`echo "$result"  | grep "DBTYPE:" | cut -d : -f 2`
echo "$CDB"
#set +x
}

########################################################################################
# Function: db_get_pdb_status
# Description: This function is responsible for determining the status of the PDB's in a
# CDB context. If PDB status is "UNUSABLE" then an alert mail will be sent
# to the user with the configured email id in the package configuration file.
# Prerequisite: Database instance should be up and running to determine the
# status of pluggable database.
########################################################################################

function db_get_pdb_status
{
  stat=$1
  result=$( $CMD $ORACLE_ADMIN -c "/bin/bash" <<EOF
  export ORACLE_HOME=${ORACLE_HOME}
  export ORACLE_SID=${SID_NAME}
  if [[ -n "${stat}" ]]; then
     echo "select concat ('PDB_ID-', concat(concat(pdb_id, ':PDB_NAME-'), concat(concat(pdb_name, ':PDB_STATUS-'), status))) as PDB_DETAILS from cdb_pdbs where status = '${stat}' order by pdb_id;" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
  else
     #Printing PDB's which are in different state other than NORMAL. This is required to log the message about the
     #pluggable databases which are not in NORMAL state after startup or before closure. 
     echo "select concat ('PDB_ID-', concat(concat(pdb_id, ':PDB_NAME-'), concat(concat(pdb_name, ':PDB_STATUS-'), status))) as PDB_DETAILS from cdb_pdbs where status != 'NORMAL' order by pdb_id;" | ${ORACLE_HOME}/bin/sqlplus '/ as sysdba'
  fi
EOF
)
PDB_STATUS=`echo "$result" | grep "PDB_ID-"`
echo "${PDB_STATUS}"
}


