#!/bin/bash
# (c) Copyright 2013 Hewlett Packard Development Company, L.P.
# Version A.12.00.00
#################################################################################
# Wait for a certain time and then kill a process if it is still running
#################################################################################
# This script sleeps for a specified amount of time and then checks if a process
# with the specified PID exists.  If the process exists, then it will kill the
# process and exit.
#
# Input Parameters:
#       Parameter 1: PID - The PID of the OS process that needs to be killed
#                               after sleeping.
#       Parameter 2: Timeout - The time (in seconds) to sleep before checking for
#                               the existence of the process.
#
# Return value:
#       0 = If process does not exist.
#       1 = If process exists and killed.
#
# REVISION HISTORY:
#     Version   mm/dd/yyyy   Changes
#     A.05.01   04/30/2012   1) Added Code to support database hang detection.
#################################################################################

#Main

PID=$1
Timeout=$2
exit_status=0
lid=""

if [[ $Timeout -ge 0 ]]; then
	sleep $Timeout;

	# Check if the process with PID exists
	lid=`UNIX95=1 ps -p $PID -o pid | grep -v PID`

	if [[ ${lid} = "$PID" ]]; then
		log_message "The database hang check script is not responding.  Killing the process"
		kill -s SIGKILL $PID;
		exit_status=1;
	fi
fi

return $exit_status;
