#!/bin/bash
# (c) Copyright 2007-2014 Hewlett Packard Development Company, L.P.
# Version A.12.00.20
################################################################################
# Script that enables/disables and starts/stops a application Servers for
# Serviceguard. (Known as Toolkit Interface Script.)
#
# This script has two parameters passed to it, which then
# will be passed to the HA server main script:
#
# The first parameter can take the following values
# - start   - to start the application
# - monitor - to monitor the application
# - stop    - to stop the application
# - monitor_listener - to monitor the listener process
# - hang_monitor - for database hand detection
#
# The second parameter will either be
# - auto  or
# - user
# based on the reason of shutdown of the package, i.e., based on whether
# the Oracle instance shutdown is user initiated or due to a failure of a service.
#
# This script exit on one of the following code:
#  0: on success
#  1: on fail
#
# REVISION HISTORY:
#     Version   mm/dd/yyyy   Changes
#     A.05.01   04/30/2012   1) Added Code to support database hang detection.
#     A.05.01.02 03/20/2013  1) Corrected incorrect numeric comparisons.
#     A.05.01.10 04/18/2013  1) Changes done to make string comparisons
#                               consistent.
#     A.12.00.00 09/27/2013  1) Changes to obsolete legacy style of
#                               packaging.
#     A.12.00.20 08/14/2014  1) Enhanced Error and Warning messages to support
#                               Error handling feature of Servicguard Manager.
################################################################################

################################################################################
# Script Program start from here
################################################################################
	
# source the cluster configuration file to get the serviceguard global
# environment variables

. /etc/cmcluster.conf

#This variable holds the value of the parameter which is passed to this script
ACT=$1
REASON=$2
# The purpose and value of the third parameter will vary based on the context.
PARAM3=$3

HA_SERVER=ORACLE
HA_SCRIPT="$TKIT_SCRIPT_DIR/haoracle.sh"


# Function to test the return code
function test_return
{
    if [[ $? -ne 0 ]]
    then
        echo "ERROR: $(date '+%b %e %T') - Failed to $ACT $HA_SERVER."
        exit 1
    fi
}

# Function to invoke a application toolkit
function invoke_app
{
    if [[ -x $HA_SCRIPT ]]; then
         . $HA_SCRIPT $ACT $REASON $PARAM3 
        test_return
    else
        echo -e "WARNING: Oracle toolkit - The toolkit script \"$HA_SCRIPT\" does not exist or"
        echo -e "\t is not excutable. Either install the toolkit in \"$TKIT_SCRIPT_DIR\" directory"
        echo -e "\t or use the chmod(1) command to make file $HA_SCRIPT executable."
    fi

    exit 0;
}

# Script entry point; Program start from here
    invoke_app

